@@ -1,3 +1,4 @@
+
 ##########################
 #biomaRt source code     #
 ##########################
@@ -148,15 +149,16 @@ listMarts <- function( mart = NULL, host="www.ensembl.org", path="/biomart/marts
 #################################
 
 useMart <- function(biomart, dataset, host = "www.ensembl.org", path = "/biomart/martservice", port = 80, archive = FALSE, ssl.verifypeer = TRUE, version, verbose = FALSE){
-
+  
   if(missing(biomart) && missing(version)) stop("No biomart databases specified. Specify a biomart database to use using the biomart or version argument")
   if(!missing(biomart)){ 
   if(!(is.character(biomart)))
       stop("biomart argument is no string.  The biomart argument should be a single character string")
   }
-  if(biomart == "ensembl" & host == "www.ensembl.org"){
+  if(biomart == "ensembl" & (host == "www.ensembl.org" | host == "uswest.ensembl.org")){
    biomart = "ENSEMBL_MART_ENSEMBL"
   }
+  reqHost = host
   marts=NULL
   marts=listMarts(host=host, path=path, port=port, includeHosts = TRUE, archive = archive, ssl.verifypeer = ssl.verifypeer)
   mindex = NA
@@ -178,6 +180,14 @@ useMart <- function(biomart, dataset, host = "www.ensembl.org", path = "/biomart
    if(!missing(version)) biomart = marts$biomart[mindex]
     biomart = sub(" ","%20",biomart, fixed = TRUE, useBytes = TRUE)
     mart <- new("Mart", biomart = biomart,vschema = marts$vschema[mindex], host = paste("http://",marts$host[mindex],":",marts$port[mindex],marts$path[mindex],sep=""), archive = archive)
+    if(length(grep("archive",martHost(mart)) > 0)){
+       if(length(grep(reqHost,martHost(mart))) == 0){
+        writeLines(paste("Note: requested host was redirected from ", reqHost, " to " ,martHost(mart),sep=""))
+     	writeLines("When using archived Ensembl versions this sometimes can result in connecting to a newer version than the intended Ensembl version")
+     	writeLines("Check your ensembl version using listMarts(mart)")
+    	}
+    }
+
     BioMartVersion=bmVersion(mart, verbose=verbose)
     if(martHost(mart) =="http://www.biomart.org:80/biomart/martservice"){
       if(verbose) writeLines("Using Central Repository at www.biomart.org");
@@ -186,6 +196,9 @@ useMart <- function(biomart, dataset, host = "www.ensembl.org", path = "/biomart
     if(verbose){
       writeLines(paste("BioMartServer running BioMart version:",BioMartVersion,sep=" "))
       writeLines(paste("Mart virtual schema:",martVSchema(mart),sep=" "))
+      if(length(grep(reqHost,martHost(mart))) == 0){
+        writeLines(paste("Requested host was redirected from ", reqHost, " to " ,martHost(mart),sep=""))
+      } 
       writeLines(paste("Mart host:",martHost(mart),sep=" "))
     }
     if(!missing(dataset)){
@@ -375,7 +388,7 @@ filterType <- function(filter, mart){
 #getBM: generic BioMart query function   # 
 ##########################################
 
-getBM <- function(attributes, filters = "", values = "", mart, curl = NULL, checkFilters = TRUE, verbose=FALSE, uniqueRows=TRUE, bmHeader=FALSE){
+getBM <- function(attributes, filters = "", values = "", mart, curl = NULL, checkFilters = TRUE, verbose=FALSE, uniqueRows=TRUE, bmHeader=FALSE, quote="\""){
   
   martCheck(mart)
   if(missing( attributes ))
@@ -526,7 +539,7 @@ getBM <- function(attributes, filters = "", values = "", mart, curl = NULL, chec
 
     ## convert the serialized table into a dataframe
     con = textConnection(postRes)
-    result = read.table(con, sep="\t", header=bmHeader, quote = "\"", comment.char = "", check.names = FALSE, stringsAsFactors=FALSE)
+    result = read.table(con, sep="\t", header=bmHeader, quote = quote, comment.char = "", check.names = FALSE, stringsAsFactors=FALSE)
     if(verbose){
       writeLines("#################\nParsed results:")
       print(result)
